@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2005-2021, PyInstaller Development Team.
+# Copyright (c) 2005-2022, PyInstaller Development Team.
 #
 # Distributed under the terms of the GNU General Public License (version 2
 # or later) with exception for distributing the bootloader.
@@ -166,7 +166,7 @@ def data_dir(
     return _data_dir_copy(request, name, tmpdir)
 
 
-class AppBuilder(object):
+class AppBuilder:
     def __init__(self, tmpdir, request, bundle_mode):
         self._tmpdir = tmpdir
         self._request = request
@@ -174,6 +174,7 @@ class AppBuilder(object):
         self._specdir = str(tmpdir)
         self._distdir = str(tmpdir / 'dist')
         self._builddir = str(tmpdir / 'build')
+        self._is_spec = False
 
     def test_spec(self, specfile, *args, **kwargs):
         """
@@ -182,6 +183,7 @@ class AppBuilder(object):
         __tracebackhide__ = True
         specfile = os.path.join(_get_spec_dir(self._request), specfile)
         # 'test_script' should handle .spec properly as script.
+        self._is_spec = True
         return self.test_script(specfile, *args, **kwargs)
 
     def test_source(self, source, *args, **kwargs):
@@ -236,7 +238,8 @@ class AppBuilder(object):
             app_args = []
 
         if app_name:
-            pyi_args.extend(['--name', app_name])
+            if not self._is_spec:
+                pyi_args.extend(['--name', app_name])
         else:
             # Derive name from script name.
             app_name = os.path.splitext(os.path.basename(script))[0]
@@ -251,7 +254,7 @@ class AppBuilder(object):
         if not self._test_building(args=pyi_args):
             pytest.fail('Building of %s failed.' % script)
 
-        marker('Build finshed, now running executable.')
+        marker('Build finished, now running executable.')
         self._test_executables(app_name, args=app_args, runtime=runtime, run_from_path=run_from_path, **kwargs)
         marker('Running executable finished.')
 
@@ -350,12 +353,12 @@ class AppBuilder(object):
         # Using sys.stdout/sys.stderr for subprocess fixes printing messages in Windows command prompt. Py.test is then
         # able to collect stdout/sterr messages and display them if a test fails.
         for _ in range(_MAX_RETRIES):
-            retcode = self.__run_executable(args, exe_path, prog_env, prog_cwd, runtime)
+            retcode = self._run_executable_(args, exe_path, prog_env, prog_cwd, runtime)
             if retcode != 1:  # retcode == 1 means a timeout
                 break
         return retcode
 
-    def __run_executable(self, args, exe_path, prog_env, prog_cwd, runtime):
+    def _run_executable_(self, args, exe_path, prog_env, prog_cwd, runtime):
         process = psutil.Popen(
             args, executable=exe_path, stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=prog_env, cwd=prog_cwd
         )
@@ -398,27 +401,34 @@ class AppBuilder(object):
 
         :param args: additional CLI options for PyInstaller.
 
-        Return True if build succeded False otherwise.
+        Return True if build succeeded False otherwise.
         """
-        default_args = [
-            '--debug=bootloader',
-            '--noupx',
-            '--specpath', self._specdir,
-            '--distpath', self._distdir,
-            '--workpath', self._builddir,
-            '--path', _get_modules_dir(self._request),
-            '--log-level=INFO',
-        ]   # yapf: disable
+        if self._is_spec:
+            default_args = [
+                '--distpath', self._distdir,
+                '--workpath', self._builddir,
+                '--log-level=INFO',
+            ]  # yapf: disable
+        else:
+            default_args = [
+                '--debug=bootloader',
+                '--noupx',
+                '--specpath', self._specdir,
+                '--distpath', self._distdir,
+                '--workpath', self._builddir,
+                '--path', _get_modules_dir(self._request),
+                '--log-level=INFO',
+            ]  # yapf: disable
 
-        # Choose bundle mode.
-        if self._mode == 'onedir':
-            default_args.append('--onedir')
-        elif self._mode == 'onefile':
-            default_args.append('--onefile')
-        # if self._mode is None then just the spec file was supplied.
+            # Choose bundle mode.
+            if self._mode == 'onedir':
+                default_args.append('--onedir')
+            elif self._mode == 'onefile':
+                default_args.append('--onefile')
+            # if self._mode is None then just the spec file was supplied.
 
         pyi_args = [self.script] + default_args + args
-        # TODO: fix return code in running PyInstaller programatically.
+        # TODO: fix return code in running PyInstaller programmatically.
         PYI_CONFIG = configure.get_config(upx_dir=None)
         # Override CACHEDIR for PyInstaller and put it into self.tmpdir
         PYI_CONFIG['cachedir'] = str(self._tmpdir)
@@ -554,3 +564,18 @@ def compiled_dylib(tmpdir, request):
         old_wd.chdir()
 
     return tmp_data_dir
+
+
+@pytest.fixture
+def pyi_windowed_builder(pyi_builder: AppBuilder):
+    """A pyi_builder equivalent for testing --windowed applications."""
+
+    # psutil.Popen() somehow bypasses an application's windowed/console mode so that any application built in
+    # --windowed mode but invoked with psutil still receives valid std{in,out,err} handles and behaves exactly like
+    # a console application. In short, testing windowed mode with psutil is a null test. We must instead use subprocess.
+
+    def _run_executable_(args, exe_path, prog_env, prog_cwd, runtime):
+        return subprocess.run([exe_path, *args], env=prog_env, cwd=prog_cwd, timeout=runtime).returncode
+
+    pyi_builder._run_executable_ = _run_executable_
+    yield pyi_builder
